package jp.co.ecosysnetwork.ccloca_app.activity;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.DialogFragment;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Xml;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;

import org.xmlpull.v1.XmlPullParser;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Locale;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.ccloca_app.NoticeDialogFragment;
import jp.co.ecosysnetwork.ccloca_app.R;
import jp.co.ecosysnetwork.ccloca_app.models.busRoute_infos;
import jp.co.ecosysnetwork.ccloca_app.models.course_settings;
import jp.co.ecosysnetwork.ccloca_app.models.system_settings;
import jp.co.ecosysnetwork.universal.EsnDebugLog;
import jp.co.ecosysnetwork.universal.EsnUtility;
import jp.co.ecosysnetwork.universal.EsnXmlParser;

/**
 * システム設定アクティビティ
 */
public class SystemSettingsActivity
        extends BaseActivity
        implements View.OnClickListener,
                    NoticeDialogFragment.NoticeDialogListener {
    //region 定数
    /**
     * 保存確認ダイアログ
     */
    private static final String DIALOG_TAG_SAVE = "cfmSave";
    /**
     * 認証確認ダイアログ
     */
    private static final String DIALOG_TAG_AUTH = "cfmAuth";
    /**
     * 認証成功ダイアログ
     */
    private static final String DIALOG_TAG_AUTH_SUCCESS = "AuthSuccess";
    /**
     * 認証成功の保存失敗ダイアログ
     */
    private static final String DIALOG_TAG_AUTH_SUCCESS_SAVE_ERROR = "AuthSuccessSaveError";
    /**
     * 認証失敗ダイアログ
     */
    private static final String DIALOG_TAG_AUTH_ERROR = "AuthError";
    /**
     * 携帯端末認証送信後の待機時間
     */
    private static final int WAIT_MSEC_USER_AUTH_SEND = 1000;
    //endregion
    //region メンバ変数
    /**
     * サーバURLエディットテキスト
     */
    private EditText _edtServerUrl;
    /**
     * 更新間隔エディットテキスト
     */
    private EditText _edtUpdateCycleMsec;
    /**
     * システム設定保存ボタン
     */
    private Button _btnSave;
    /**
     * 携帯端末IDエディットテキスト
     */
    private EditText _edtPhoneId;
    /**
     * 携帯端末認証ボタン
     */
    private Button _btnAuth;
    /**
     * 携帯端末認証プログレスバー
     */
    private ProgressBar _prgAuth;
    /**
     * 保存元の値
     */
    private system_settings _org_model;
    /**
     * 編集中の値
     */
    private system_settings _tmp_model;
    /**
     * 通信中フラグ
     */
    private boolean _communicating;
    /**
     * コース設定モデル
     */
    private course_settings _course_settings;
    /**
     * バス経路情報List
     */
    private busRoute_infos _busRoute_infos;
    //endregion
    //region 内部メソッド
    /**
     * リフレッシュ（ビュー）
     */
    private void refreshView() {
        // サーバURL
        _edtServerUrl.setText(_tmp_model.server_url);

        // 携帯端末ID
        _edtPhoneId.setText(_tmp_model.phone_id);
    }
    /**
     * リフレッシュ（ボタン）
     */
    private void refreshButton() {
        // 保存ボタン
        _btnSave.setEnabled(!_communicating &&
                            !_org_model.equals(_tmp_model, system_settings.SAVE_ITEM.EXC_USER_ID));

        // 認証ボタン
        boolean isEnableAuth = !_communicating &&
                                (!_org_model.equals(_tmp_model, system_settings.SAVE_ITEM.ONLY_PHONE_ID));
        _btnAuth.setEnabled(isEnableAuth);
    }
    /**
     * 終了処理
     */
    private void terminate() {
    }
    /**
     * 戻るボタン押下
     */
    private void clickBtnBack() {
        EsnDebugLog.outputLog("戻るボタン押下", EsnDebugLog.DIV.INFO);

        terminate();
        finishAndRemoveTask();
    }
    /**
     * 保存確認ダイアログ：はいボタン押下
     */
    private void clickPosCfmSave() {
        // システム設定の保存処理
        if (_tmp_model.savePreference(system_settings.SAVE_ITEM.EXC_USER_ID)) {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN,"システム設定の保存処理成功 %s", _tmp_model.toLogString()), EsnDebugLog.DIV.WARN);

            // リフレッシュ
            _tmp_model.copy(_org_model, system_settings.SAVE_ITEM.EXC_USER_ID);
            refreshButton();
        }
        else {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN,"システム設定の保存処理失敗 %s", _tmp_model.toLogString()), EsnDebugLog.DIV.ERR);
        }
    }
    /**
     *  保存ボタン押下
     */
    private void clickBtnSave() {
        EsnDebugLog.outputLog("保存ボタン押下", EsnDebugLog.DIV.INFO);

        // 変更データがない場合は処理しない
        if (_org_model.equals(_tmp_model, system_settings.SAVE_ITEM.EXC_USER_ID)) {
            EsnDebugLog.outputLog("変更データが存在しないため保存しない", EsnDebugLog.DIV.WARN);
            return;
        }

        // 保存確認ダイアログ表示
        DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                getString(R.string.msgSystemSettingSave),
                false,
                false,
                getString(R.string.yes),
                getString(R.string.no)
        );
        newFragment.show(getSupportFragmentManager(), DIALOG_TAG_SAVE);
    }
    /**
     * 認証確認ダイアログ：はいボタン押下
     */
    private void clickPosCfmAuth() {
        // プログレスバー表示
        _communicating = true;
        refreshButton();
        _prgAuth.setVisibility(View.VISIBLE);

        final Handler handler = new Handler(Looper.getMainLooper());
        ExecutorService executor = Executors.newSingleThreadExecutor();
        executor.execute(new Runnable() {
            /**
             * 携帯端末認証バックグラウンド処理
             */
            @Override
            public void run() {
                final String ret = userAuthExec();
                handler.postDelayed(new Runnable() {
                    /**
                     * バックグラウンド終了後のUI処理
                     */
                    @Override
                    public void run() {
                        // プログレスバー消す
                        _communicating = false;
                        _prgAuth.setVisibility(View.GONE);
                        if (ret == null) {
                            // 認証成功
                            if (_tmp_model.savePreference(system_settings.SAVE_ITEM.ONLY_PHONE_ID)) {
                                EsnDebugLog.outputLog(String.format(Locale.JAPAN,"携帯端末IDの認証処理成功 %s", _tmp_model.toLogString()), EsnDebugLog.DIV.WARN);

                                // リフレッシュ
                                _tmp_model.copy(_org_model, system_settings.SAVE_ITEM.ONLY_PHONE_ID);
                                _course_settings.initSelectedCourse();
                                _busRoute_infos.initBusRouteInfos();

                                // 認証成功ダイアログ
                                DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                                        getString(R.string.msgSuccessUserAuth),
                                        false,
                                        false,
                                        getString(R.string.ok),
                                        "");
                                newFragment.show(getSupportFragmentManager(), DIALOG_TAG_AUTH_SUCCESS);
                            }
                            else {
                                EsnDebugLog.outputLog(String.format(Locale.JAPAN,"携帯端末IDの保存処理失敗 %s", _tmp_model.toLogString()), EsnDebugLog.DIV.ERR);

                                // 認証成功後の携帯端末ID保存失敗ダイアログ
                                DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleError),
                                        getString(R.string.msgSuccessUserAuthButSaveError),
                                        false,
                                        false,
                                        getString(R.string.ok),
                                        "");
                                newFragment.show(getSupportFragmentManager(), DIALOG_TAG_AUTH_SUCCESS_SAVE_ERROR);
                            }
                        }
                        else {
                            // 認証失敗ダイアログ
                            DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleError),
                                    ret,
                                    false,
                                    false,
                                    getString(R.string.ok),
                                    "");
                            newFragment.show(getSupportFragmentManager(), DIALOG_TAG_AUTH_ERROR);
                        }
                        // リフレッシュ
                        refreshView();
                        refreshButton();
                    }
                }, 500);
            }
        });
    }
    /**
     *  認証ボタン押下
     */
    private void clickBtnAuth() {
        EsnDebugLog.outputLog("認証ボタン押下", EsnDebugLog.DIV.INFO);

        // 変更データがない場合は処理しない
        if (_org_model.equals(_tmp_model, system_settings.SAVE_ITEM.ONLY_PHONE_ID)) {
            EsnDebugLog.outputLog("変更データが存在しないため認証しない", EsnDebugLog.DIV.WARN);
            return;
        }

        // 認証確認ダイアログ表示
        DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                getString(R.string.msgUserAuth),
                false,
                false,
                getString(R.string.yes),
                getString(R.string.no)
        );
        newFragment.show(getSupportFragmentManager(), DIALOG_TAG_AUTH);
    }
    //region 携帯端末認証処理
    /**
     * 携帯端末認証実行処理
     * @return エラーメッセージ
     */
    private String userAuthExec(){
        String ret = null;

        HttpURLConnection con = null;
        DataOutputStream os = null;
        InputStream is = null;
        try {
            URL accUrl = new URL(_tmp_model.server_url + MyApplication.APIURL_CHECK_PHONE_ID);
            con = (HttpURLConnection)accUrl.openConnection();
            con.setRequestMethod("POST");
            con.setDoOutput(true);
            con.setDoInput(true);
            con.setUseCaches(false);

            // ヘッダ設定
            con.setRequestProperty("ANDROID-API-KEY", MyApplication.API_KEY);
            con.setRequestProperty("Connection", "Keep-Alive");
            con.setRequestProperty("Charset", "UTF-8");
            String boundary = "-----------------------------" + System.currentTimeMillis();
            con.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
            con.connect();

            // 送信データの設定
            os = new DataOutputStream(con.getOutputStream());
            append_string(os, boundary, "intFacilityPhoneNo", _tmp_model.phone_id);
            append_final(os, boundary);
            os = null;

            // 携帯端末認証送信後のウェイト
            MyApplication.sleep(WAIT_MSEC_USER_AUTH_SEND);

            // 受信データを取得
            is = con.getInputStream();

            // 受信データのXMLパース
            XmlPullParser parser = Xml.newPullParser();
            parser.setInput(is, "UTF-8");

            // 受信データのパース
            EsnXmlParser esnParser = new EsnXmlParser(parser);
            ret = esnParser.tryParse();
            if (ret == null) {
                // パース成功
                ret = getErrorMessage("4");   // 携帯端末ID不明
                for (EsnXmlParser.ParseItem item : esnParser.getParseItems()) {
                    if (item.tag.equals("intFacilityPhoneNo")) {
                        // 携帯端末IDを更新
                        _tmp_model.phone_id = item.value;
                        ret = null;      // 認証成功
                    }
                    else if (item.tag.equals("intUpdateInterval")) {
                        // 更新頻度
                        _tmp_model.update_cycle_msec = Integer.parseInt(item.value);
                    }
                    else if (item.tag.equals("errorMsg")) {
                        // エラーメッセージ設定
                        ret = item.value;
                    }
                }
            }else
            {
                ret = "データの変換に失敗しました。";
            }
        } catch (Exception e1) {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN, "例外エラー１ %s", e1.toString()), EsnDebugLog.DIV.ERR);
            ret = getString(R.string.msgCommunicationError);
        } finally {
            try {
                if (is != null) is.close();
                if (os != null) os.close();
                if (con != null) con.disconnect();
            } catch (Exception e2) {
                EsnDebugLog.outputLog(String.format(Locale.JAPAN, "例外エラー２ %s", e2.toString()), EsnDebugLog.DIV.ERR);
                ret = getString(R.string.msgCommunicateCmpltError);
            }
        }
        return ret;
    }
    /**
     * エラーメッセージの取得
     * @param codeValue　codeタグの値
     * @return エラーメッセージ
     */
    private String getErrorMessage(String codeValue) {
        String ret = null;
        int tmpValue = -1;
        if (EsnUtility.isNumeric(codeValue)) {
            tmpValue = Integer.parseInt(codeValue);
        }
        switch (tmpValue) {
            case 0: break;  // 正常
            case 1: ret = getString(R.string.commErrmsg01); break;
            case 2: ret = getString(R.string.commErrmsg02); break;
            case 3: ret = getString(R.string.commErrmsg03); break;
            case 4: ret = getString(R.string.commErrmsg04); break;
            case 5: ret = getString(R.string.commErrmsg05); break;
            case 6: ret = getString(R.string.commErrmsg06); break;
            case 7: ret = getString(R.string.commErrmsg07); break;
            case 8: ret = getString(R.string.commErrmsg08); break;
            case 9: ret = getString(R.string.commErrmsg09); break;
            case 10: ret = getString(R.string.commErrmsg10); break;
            case 11: ret = getString(R.string.commErrmsg11); break;
            case 12: ret = getString(R.string.commErrmsg12); break;
            case 13: ret = getString(R.string.commErrmsg13); break;
            case 14: ret = getString(R.string.commErrmsg14); break;
            case 15: ret = getString(R.string.commErrmsg15); break;
            default: ret = getString(R.string.commErrmsgOT); break;
        }
        return ret;
    }
    //endregion
    //endregion
    //region イベントメソッド
    /**
     * アクティビティ作成イベント
     *
     * @param savedInstanceState アクティビティが以前にシャットダウンされた後に再初期化されている場合、このバンドルにはonSaveInstanceState（Bundle）で最後に提供されたデータが含まれます。 注：それ以外の場合はnullです。
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_system_settings);

        // メンバ変数設定
        _edtServerUrl = findViewById(R.id.edtServerUrl);
        _btnSave = findViewById(R.id.btnSave);
        _edtPhoneId = findViewById(R.id.edtUserId);
        _btnAuth = findViewById(R.id.btnAuth);
        _prgAuth = findViewById(R.id.prgAuth);
        _org_model = new system_settings();
        _tmp_model = _org_model.clone();
        _course_settings = new course_settings();
        _busRoute_infos = new busRoute_infos();
        _communicating = false;

        // ボタンのリスナ登録
        findViewById(R.id.imbBack).setOnClickListener(this);
        _btnSave.setOnClickListener(this);
        _btnAuth.setOnClickListener(this);

        // サーバURLエディットテキストのリスナ登録
        _edtServerUrl.addTextChangedListener(new TextWatcher() {
            @Override public void beforeTextChanged(CharSequence s, int start, int count, int after) { /*テキスト変更前*/ }
            @Override public void onTextChanged(CharSequence s, int start, int before, int count) { /* テキスト変更中 */ }
            @Override public void afterTextChanged(Editable s) { /* テキスト変更後 */
                _tmp_model.server_url = s.toString();
                refreshButton();
            }
        });

        // 携帯端末IDエディットテキストのリスナ登録
        _edtPhoneId.addTextChangedListener(new TextWatcher() {
            @Override public void beforeTextChanged(CharSequence s, int start, int count, int after) { /*テキスト変更前*/ }
            @Override public void onTextChanged(CharSequence s, int start, int before, int count) { /* テキスト変更中 */ }
            @Override public void afterTextChanged(Editable s) { /* テキスト変更後 */
                _tmp_model.phone_id = s.toString();
                refreshButton();
            }
        });

        // リフレッシュ
        refreshView();
        refreshButton();
    }
    /**
     * クリックリスナー
     * @param view 対象のビュー
     */
    public void onClick(View view) {
        Intent intent;
        switch (view.getId()) {
            case R.id.imbBack: clickBtnBack(); break;   // 戻る
            case R.id.btnSave: clickBtnSave(); break; // 保存ボタン
            case R.id.btnAuth: clickBtnAuth(); break; // 認証ボタン
        }
    }
    /**
     * ダイアログのOKボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogPositiveClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "POSSTIVEボタン押下 %s", tag), EsnDebugLog.DIV.INFO);

        switch (tag) {
            case DIALOG_TAG_SAVE:
                // 保存確認ダイアログではいボタン押下
                clickPosCfmSave();
                break;
            case DIALOG_TAG_AUTH:
                // 認証確認ダイアログではいボタン押下
                clickPosCfmAuth();
                break;
        }
    }
    /**
     * ダイアログのキャンセルボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogNegativeClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "NEGATIVEボタン押下 %s", tag), EsnDebugLog.DIV.INFO);
    }
    /**
     * ダイアログのニュートラルボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogNeutralClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "NEUTRALボタン押下 %s", tag), EsnDebugLog.DIV.INFO);
    }
    //endregion
}