package jp.co.ecosysnetwork.ccloca_app.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.List;

import jp.co.ecosysnetwork.ccloca_app.models.busRoute_info;

public class BusRouteAdapter extends ArrayAdapter<busRoute_info>
{
    //region メンバ変数
    /**
     * バスコース情報リスト
     */
    private List<busRoute_info> _items;
    /**
     * 選択コース
     */
    private int _selected_key;
    /**
     * 自身
     */
    private BusRouteAdapter _self;
    /**
     * コンテキスト
     */
    private Context _context;
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     *
     * @param context   コンテキスト
     * @param items     リストビューの要素
     */
    public BusRouteAdapter(Context context, List<busRoute_info> items)
    {
        super(context,0, items);
        _context = context;
        _items = items;
        _selected_key = 0;
        _self = this;
    }
    //endregion
    //region    イベントメソッド
    @NonNull
    @Override
    public View getView(final int position, @Nullable View convertView,  @NonNull final ViewGroup parent)
    {
        TextView textView = new TextView(_context);
        textView.setTextSize(25);
        busRoute_info data = _items.get(position);
        if (data != null) {
            textView.setText(data.bus_route_name);
        }
        return textView;
    }
    @NonNull
    @Override
    public View getDropDownView(int position, View convertView, @NonNull final ViewGroup parent) {
        TextView textView = new TextView(_context);
        textView.setTextSize(25);
        if (position <= this.getCount()) {
            busRoute_info data = _items.get(position);
            if (data != null) {
                textView.setText(data.bus_route_name);
            }
        }
        return textView;
    }
    /**
     * 要素数取得
     */
    @Override
    public int getCount() {
        return _items.size();
    }
    /**
     * 選択キーを設定
     * @param info_id コースキー
     */
    public void setSelectedKey(int info_id) {
        _selected_key = info_id;
        // 描画更新
        _self.notifyDataSetChanged();
    }
    /**
     * 選択キーを取得
     * @return コースキー
     */
    public int getSelectedKey() {
        return _selected_key;
    }
    /**
     * コース情報リストの設定
     * @param items コース情報リスト
     */
    public void setItems(List<busRoute_info> items) {
        _items = items;
        // 描画更新
        notifyDataSetChanged();
    }
    //endregion
}
