package jp.co.ecosysnetwork.ccloca_app.models;

import android.content.Context;
import android.content.SharedPreferences;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.universal.EsnDebugLog;

/**
 * コース設定モデル
 */
public class course_settings implements Cloneable {
    //region 外部公開メンバ変数
    /**
     * 選択中コース情報モデル
     */
    public busRoute_info selected_busRoute_info;
    /**
     * 選択中コース情報モデルポジション
     */
    public int selected_position;
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     */
    public course_settings() {
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);
        selected_busRoute_info = new busRoute_info();
        selected_busRoute_info.bus_route_no = pref.getString(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_ID, "0");
        selected_busRoute_info.bus_route_name = pref.getString(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_NAME, "コースを選択してください。");
        selected_position = pref.getInt(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_POSITION, 0);
    }
    //endregion
    //region 外部公開メソッド
    /**
     * デバッグログ文字列の取得
     * @return デバッグログ文字列
     */
    public String toLogString() {
        return String.format(Locale.JAPAN," BusRouteId = %s\n BusRouteName = %s\n Position = %d", selected_busRoute_info.bus_route_no, selected_busRoute_info.bus_route_name, selected_position);
    }
    /**
     * 自身のクローンを返す
     * @return 自身のクローン
     */
    @Override
    public course_settings clone() {
        course_settings ret = null;
        try {
            ret = (course_settings)super.clone();
            ret.selected_busRoute_info = this.selected_busRoute_info;
            ret.selected_position = this.selected_position;
        } catch (CloneNotSupportedException ex) {
            EsnDebugLog.outputLog("clone失敗 " + ex.toString(), EsnDebugLog.DIV.ERR);
        }
        return ret;
    }
    /**
     * 同値判定（Android Studioの自動生成機能を使用）
     * @param o 比較対象オブジェクト
     * @return true：同じ false：異なる
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        course_settings that = (course_settings) o;
        return (selected_busRoute_info.equals(that.selected_busRoute_info) && (selected_position == that.selected_position));
    }
    /**
     * プリファレンスへの保存
     * @return　true：成功　false：失敗
     */
    public boolean savePreference() {
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();

        editor.putString(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_ID, selected_busRoute_info.bus_route_no);
        editor.putString(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_NAME, selected_busRoute_info.bus_route_name);
        editor.putInt(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_POSITION, selected_position);
        return editor.commit();
    }
    /**
     * 設定コースの初期化
     * @return　true：成功　false：失敗
     */
    public boolean initSelectedCourse() {
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();

        editor.putString(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_ID, "0");
        editor.putString(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_NAME, "コースを選択してください");
        editor.putInt(MyApplication.PKEY_SELECTED_BUS_ROUTE_INFO_POSITION, 0);
        return editor.commit();
    }
    //endregion
}
