package jp.co.ecosysnetwork.ccloca_app.models;

import android.content.Context;
import android.content.SharedPreferences;

import java.util.Locale;
import java.util.Objects;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.universal.EsnDebugLog;

/**
 * システム設定モデル
 */
public class system_settings implements Cloneable {
    //region 列挙体
    /**
     * 保存項目
     */
    public enum SAVE_ITEM {
        /**
         * 全て
         */
        ALL,
        /**
         * 携帯端末ID以外
         */
        EXC_USER_ID,
        /**
         * 携帯端末IDのみ
         */
        ONLY_PHONE_ID
    }
    //endregion
    //region 外部公開メンバ変数
    /**
     * サーバURL
     */
    public String server_url;
    /**
     * 更新間隔
     */
    public int update_cycle_msec;
    /**
     * 携帯端末ID
     */
    public String phone_id;
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     */
    public system_settings() {
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);
        server_url = pref.getString(MyApplication.PKEY_SERVER_URL, "http://10.0.2.2/bus");
        update_cycle_msec = pref.getInt(MyApplication.PKEY_UPDATE_CYCLE_MSEC, 10000);
        phone_id = pref.getString(MyApplication.PKEY_PHONE_ID, "");
    }
    //endregion
    //region 外部公開メソッド
    /**
     * デバッグログ文字列の取得
     * @return デバッグログ文字列
     */
    public String toLogString() {
        StringBuilder ret = new StringBuilder();
        ret.append(String.format(Locale.JAPAN," server_url=%s\n", server_url));
        ret.append(String.format(Locale.JAPAN," update_cycle_msec=%d\n", update_cycle_msec));
        ret.append(String.format(Locale.JAPAN," phone_id=%s\n", phone_id));
        return ret.toString();
    }
    /**
     * 自身のクローンを返す
     * @return 自身のクローン
     */
    @Override
    public system_settings clone() {
        system_settings ret = null;
        try {
            ret = (system_settings)super.clone();
        } catch (CloneNotSupportedException ex) {
            EsnDebugLog.outputLog("clone失敗 " + ex.toString(), EsnDebugLog.DIV.ERR);
        }
        return ret;
    }
    /**
     * コピー処理
     * @param to コピー先
     * @param saveItem 保存項目
     */
    public void copy(system_settings to, SAVE_ITEM saveItem) {
        // サーバURL
        if ((saveItem == SAVE_ITEM.ALL) || (saveItem == SAVE_ITEM.EXC_USER_ID)) {
            to.server_url = new String(this.server_url);
        }

        // 更新間隔
        if ((saveItem == SAVE_ITEM.ALL) || (saveItem == SAVE_ITEM.EXC_USER_ID)) {
            to.update_cycle_msec = this.update_cycle_msec;
        }

        // 携帯端末ID
        if ((saveItem == SAVE_ITEM.ALL) || (saveItem == SAVE_ITEM.ONLY_PHONE_ID)) {
            to.phone_id = new String(this.phone_id);
        }
    }
    /**
     * 同値判定（Android Studioの自動生成機能を使用）
     * @param o 比較対象オブジェクト
     * @return true：同じ false：異なる
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        system_settings that = (system_settings) o;
        return update_cycle_msec == that.update_cycle_msec &&
                Objects.equals(server_url, that.server_url) &&
                Objects.equals(phone_id, that.phone_id)
                ;
    }
    /**
     * 同値判定（Android Studioの自動生成機能を使用）
     * @param o 比較対象オブジェクト
     * @param saveItem 保存項目
     * @return true：同じ false：異なる
     */
    public boolean equals(Object o, SAVE_ITEM saveItem) {
        if (saveItem == SAVE_ITEM.ALL) {
            return this.equals(o);
        }
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        system_settings that = (system_settings) o;

        // サーバURL
        if (saveItem == SAVE_ITEM.EXC_USER_ID) {
            if (!Objects.equals(server_url, that.server_url)) {
                return false;
            }
        }

        // 更新間隔
        if (saveItem == SAVE_ITEM.EXC_USER_ID) {
            if (update_cycle_msec != that.update_cycle_msec) {
                return false;
            }
        }

        // 携帯端末ID
        if (saveItem == SAVE_ITEM.ONLY_PHONE_ID) {
            if (!Objects.equals(phone_id, that.phone_id)) {
                return false;
            }
        }

        return true;
    }
    /**
     * ハッシュコードを取得（Android Studioの自動生成機能を使用）
     * @return ハッシュコード
     */
    @Override
    public int hashCode() {
        return Objects.hash(server_url, update_cycle_msec, phone_id);
    }
    /**
     * プリファレンスへの保存
     * @param saveItem 保存項目
     * @return　true：成功　false：失敗
     */
    public boolean savePreference(SAVE_ITEM saveItem) {
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();

        // サーバURL
        if ((saveItem == SAVE_ITEM.ALL) || (saveItem == SAVE_ITEM.EXC_USER_ID)) {
            editor.putString(MyApplication.PKEY_SERVER_URL, server_url);
        }

        // 更新間隔
        if ((saveItem == SAVE_ITEM.ALL) || (saveItem == SAVE_ITEM.EXC_USER_ID)) {
            editor.putInt(MyApplication.PKEY_UPDATE_CYCLE_MSEC, update_cycle_msec);
        }

        // 携帯端末ID
        if ((saveItem == SAVE_ITEM.ALL) || (saveItem == SAVE_ITEM.ONLY_PHONE_ID)) {
            editor.putString(MyApplication.PKEY_PHONE_ID, phone_id);
        }

        return editor.commit();
    }
    //endregion
}
